@@ -147,13 +147,18 @@ You can also supply names and values to the template at this stage:
 
     return $template->output('name' => 'value', 'name2' => 'value2');
 
-Before the template output is generated, your application's
-C<< $self->template_pre_process >> method is called.
-This method is passed a reference to the C<$template> object.
+Before the template output is generated, the C<< template_pre_process >>
+hook is called.  Any callbacks that you register to this hook will be
+called before each template is processed.  Register a
+C<template_pre_process> callback as follows:
 
-It can modify the parameters passed into the template by using the C<param> method:
+    $self->add_callback('template_pre_process', \&my_tmpl_pre_process);
 
-    sub template_pre_process {
+Pre-process callbacks will be passed a reference to the C<$template>
+object, and can can modify the parameters passed into the template by
+using the C<param> method:
+
+    sub my_tmpl_pre_process {
         my ($self, $template) = @_;
 
         # Change the internal template parameters by reference
@@ -169,18 +174,25 @@ It can modify the parameters passed into the template by using the C<param> meth
     }
 
 
-After the template output is generated, your application's
-C<< $self->template_pre_process >> method is called.
-This method is passed a reference to the template object and a reference
-to the output generated by the template.  You can modify this output:
+After the template output is generated, the C<template_post_process> hook is called.
+You can register a C<template_post_process> callback as follows:
 
-    sub template_post_process {
+    $self->add_callback('template_post_process', \&my_tmpl_post_process);
+
+Any callbacks that you register to this hook will be called after each
+template is processed, and will be passed both a reference to the
+template object and a reference to the output generated by the template.
+This allows you to modify the output of the template:
+
+    sub my_tmpl_post_process {
         my ($self, $template, $output_ref) = @_;
 
         $$output_ref =~ s/foo/bar/;
     }
 
 
+
+
 When you call the C<output> method, any components embedded in the
 template are run.  See C<EMBEDDED COMPONENTS>, below.
 
@@ -200,16 +212,18 @@ sub output {
 
     $self->param(@_);
 
-    if ($self->{'webapp'} and UNIVERSAL::can($self->{'webapp'}, 'template_pre_process')) {
-        $self->{'webapp'}->template_pre_process($self);
+    my $webapp = $self->{'webapp'};
+
+    if ($webapp and $webapp->can('call_hook')) {
+        $webapp->call_hook('template_pre_process', $self);
     }
 
     my $output = $self->render_template;
 
-    if ($self->{'webapp'} and UNIVERSAL::can($self->{'webapp'}, 'template_post_process')) {
+    if ($webapp and $webapp->can('call_hook')) {
         my $output_param = $output;
         $output_param = \$output_param unless ref $output_param;
-        $self->{'webapp'}->template_post_process($output_param);
+        $webapp->call_hook('template_post_process', $output_param);
     }
     return $output;
 
@@ -377,6 +391,7 @@ sub _require_prerequisite_modules {
         if ($@) {
             push @missing_modules, $module;
         }
+        # $module->import;
     }
     if (@missing_modules) {
         foreach my $module (@missing_modules) {
@@ -404,6 +419,7 @@ be notified of progress on your bug as I make changes.
     CGI::Application::Plugin::AnyTemplate::ComponentHandler
     CGI::Application::Plugin::AnyTemplate::Driver::HTMLTemplate
     CGI::Application::Plugin::AnyTemplate::Driver::HTMLTemplateExpr
+    CGI::Application::Plugin::AnyTemplate::Driver::HTMLTemplatePluggable
     CGI::Application::Plugin::AnyTemplate::Driver::TemplateToolkit
     CGI::Application::Plugin::AnyTemplate::Driver::Petal
 
@@ -411,6 +427,10 @@ be notified of progress on your bug as I make changes.
 
     Template::Toolkit
     HTML::Template
+
+    HTML::Template::Pluggable
+    HTML::Template::Plugin::Dot
+
     Petal
 
     CGI::Application::Plugin::TT
