@@ -45,7 +45,9 @@ use Carp;
 
 use CGI::Application::Plugin::AnyTemplate::ComponentHandler;
 
-use base 'CGI::Application::Plugin::AnyTemplate::Base';
+use CGI::Application::Plugin::AnyTemplate::Base;
+use vars qw(@ISA);
+@ISA = ('CGI::Application::Plugin::AnyTemplate::Base');
 
 =head1 CONFIGURATION
 
@@ -68,6 +70,8 @@ the C<template_extension> is C<.html>.
 
 =item associate_query
 
+B<This feature is now deprecated and will be removed in a future release.>
+
 If this config parameter is true, then
 L<CGI::Application::Plugin::AnyTemplate::Driver::HTMLTemplateExpr> will
 copy all of the webapp's query params into the template using
@@ -77,7 +81,7 @@ L<HTML::Template::Expr>'s C<associate> mechanism:
         associate => $self->query,
     );
 
-By default C<associate_query> is true.
+By default C<associate_query> is false.
 
 If you provide an C<associate> config parameter of your own, that will
 disable the C<associate_query> functionality.
@@ -100,7 +104,7 @@ sub default_driver_config {
     (
         template_extension => '.html',
         embed_tag_name     => 'CGIAPP_embed',
-        associate_query    => 1,
+        associate_query    => 0,
     );
 }
 
@@ -208,6 +212,7 @@ sub render_template {
     CGI::Application::Plugin::AnyTemplate::Base
     CGI::Application::Plugin::AnyTemplate::ComponentHandler
     CGI::Application::Plugin::AnyTemplate::Driver::HTMLTemplate
+    CGI::Application::Plugin::AnyTemplate::Driver::HTMLTemplatePluggable
     CGI::Application::Plugin::AnyTemplate::Driver::TemplateToolkit
     CGI::Application::Plugin::AnyTemplate::Driver::Petal
 
@@ -215,6 +220,10 @@ sub render_template {
 
     Template::Toolkit
     HTML::Template
+
+    HTML::Template::Pluggable
+    HTML::Template::Plugin::Dot
+
     Petal
 
     Exporter::Renaming
