@@ -118,7 +118,9 @@ use Carp;
 
 use CGI::Application::Plugin::AnyTemplate::ComponentHandler;
 
-use base 'CGI::Application::Plugin::AnyTemplate::Base';
+use CGI::Application::Plugin::AnyTemplate::Base;
+use vars qw(@ISA);
+@ISA = ('CGI::Application::Plugin::AnyTemplate::Base');
 
 =head1 CONFIGURATION
 
@@ -141,6 +143,8 @@ the C<template_extension> is C<.xhtml>.
 
 =item emulate_associate_query
 
+B<This feature is now deprecated and will be removed in a future release.>
+
 If this config parameter is true, then L<CGI::Application::Plugin::AnyTemplate::Driver::TemplateToolkit>
 will copy all of the webapp's query params into the template.
 
@@ -151,7 +155,7 @@ C<associate> feature with the webapp's query object:
         associate => $self->query,
     );
 
-By default C<emulate_associate_query> is true.
+By default C<emulate_associate_query> is false.
 
 =item object_caching
 
@@ -193,7 +197,7 @@ sub default_driver_config {
         object_caching          => 1,
         template_extension      => '.tmpl',
         embed_tag_name          => 'CGIAPP',
-        emulate_associate_query => 1,
+        emulate_associate_query => 0,
     );
 }
 
@@ -321,12 +325,17 @@ sub render_template {
     CGI::Application::Plugin::AnyTemplate::ComponentHandler
     CGI::Application::Plugin::AnyTemplate::Driver::HTMLTemplate
     CGI::Application::Plugin::AnyTemplate::Driver::HTMLTemplateExpr
+    CGI::Application::Plugin::AnyTemplate::Driver::HTMLTemplatePluggable
     CGI::Application::Plugin::AnyTemplate::Driver::Petal
 
     CGI::Application
 
     Template::Toolkit
     HTML::Template
+
+    HTML::Template::Pluggable
+    HTML::Template::Plugin::Dot
+
     Petal
 
     Exporter::Renaming
